(*
  ocamldap public interface
  $Id: ocamldap.mli4,v 1.9 2004/07/19 01:53:21 eric Exp $
*)

(* 
  OCAMLDAP TYPES
*)

type conn
type msgid



and auth_method = [ `SIMPLE ]

and error_code =
  [ `SUCCESS
  | `OPERATIONS_ERROR
  | `PROTOCOL_ERROR
  | `TIMELIMIT_EXCEEDED
  | `SIZELIMIT_EXCEEDED
  | `COMPARE_FALSE
  | `COMPARE_TRUE
  | `AUTH_METHOD_NOT_SUPPORTED
  | `STRONG_AUTH_NOT_SUPPORTED
  | `STRONG_AUTH_REQUIRED
  | `PARTIAL_RESULTS
  | `REFERRAL
  | `ADMINLIMIT_EXCEEDED
  | `UNAVAILABLE_CRITICAL_EXTENSION
  | `CONFIDENTIALITY_REQUIRED
  | `SASL_BIND_IN_PROGRESS
  | `NO_SUCH_ATTRIBUTE
  | `UNDEFINED_TYPE
  | `INAPPROPRIATE_MATCHING
  | `CONSTRAINT_VIOLATION
  | `TYPE_OR_VALUE_EXISTS
  | `INVALID_SYNTAX
  | `NO_SUCH_OBJECT
  | `ALIAS_PROBLEM
  | `INVALID_DN_SYNTAX
  | `IS_LEAF
  | `ALIAS_DEREF_PROBLEM
  | `INAPPROPRIATE_AUTH
  | `INVALID_CREDENTIALS
  | `INSUFFICIENT_ACCESS
  | `BUSY
  | `UNAVAILABLE
  | `UNWILLING_TO_PERFORM
  | `LOOP_DETECT
  | `NAMING_VIOLATION
  | `OBJECT_CLASS_VIOLATION
  | `NOT_ALLOWED_ON_NONLEAF
  | `NOT_ALLOWED_ON_RDN
  | `ALREADY_EXISTS
  | `NO_OBJECT_CLASS_MODS
  | `RESULTS_TOO_LARGE
  | `AFFECTS_MULTIPLE_DSAS
  | `OTHER
  | `SERVER_DOWN
  | `LOCAL_ERROR
  | `ENCODING_ERROR
  | `DECODING_ERROR
  | `TIMEOUT
  | `AUTH_UNKNOWN
  | `FILTER_ERROR
  | `USER_CANCELLED
  | `PARAM_ERROR
  | `NO_MEMORY
  | `CONNECT_ERROR
  | `NOT_SUPPORTED
  | `CONTROL_NOT_FOUND
  | `NO_RESULTS_RETURNED
  | `MORE_RESULTS_TO_RETURN
  | `CLIENT_LOOP
  | `REFERRAL_LIMIT_EXCEEDED]
and mod_op = [ `ADD | `DELETE | `REPLACE]
and search_scope = [ `BASE | `ONELEVEL | `SUBTREE]
and attr = { attr_name : string; attr_values : string array; } 
and entry = { entry_dn : string; entry_attrs : attr list; } 
and modattr = mod_op * string * string list
and result = entry list

exception LDAP_Failure of error_code

(*
  PUBLIC INTERFACE FUNCTIONS
*)

(** Direct access to LDAP function *)

(** Adds a new entry to the ldap database. *)
val add_s : conn -> dn:string -> attr:modattr list -> unit

(** Binds to the ldap server with the given credential and auth_method. *)
val bind_s :
  ?who:string -> ?cred:string -> ?auth_method:auth_method -> conn -> unit

(** Deletes the entry with the given dn from the database. *)
val delete_s : conn -> dn:string -> unit

(** Retrieves an error message for the given ldap error code. *)
val err2string : error_code -> string

(** Initializes a connection to the given ldap server.  The connection is
 * not actually established until one of the bind functions is called. *)
val init : ?version:int -> ?port:int -> string -> conn

(* Binds using an already-established kerberos ticket.  kerberos_bind_s
performs both stages of kerberos authentication.  kerberos_bind1_s and
kerberos_bind2_s perform only the first and second stages of kerberos
authentication, respectively. *)












(** Modifies the specified entry with the modifications supplied in 'mods'. *)
val modify_s : conn -> dn:string -> mods:modattr list -> unit

(** Renames the given entry to newdn, modrdn always deletes the old rdn. *)
val modrdn_s : conn -> dn:string -> newdn:string -> unit
(** Same as modrdn but modrdn2 optionally deletes the old rdn. *)
val modrdn2_s :
  conn -> dn:string -> newdn:string -> deleteoldrdn:bool -> unit

(** Prints the given entry in LDIF format to the specified channel. *)
val print_entry : ?channel:out_channel -> entry -> unit

(** Search for the given entry with the specified base node and search scope,
 optionally limiting the returned attributes to those listed in 'attrs'. *)
val search_s :
  ?base:string ->
  ?scope:search_scope ->
  ?attrs:string list -> ?attrsonly:bool -> conn -> string -> result

(** Search for the given entry with the specified base node and search scope,
  optionally limiting the returned attributes to those listed in 'attrs'. 
  this is the asyncronis version of search, which will return the message id 
  that you must then pass to get_search_entry *)
val search :
  ?base:string ->
  ?scope:search_scope ->
  ?attrs:string list -> ?attrsonly:bool -> conn -> string -> msgid

(** actually get a search entry from and async search. If no more entries
  remain, will raise LDAP_Failure `SUCCESS. If an error occurs fetching an
  entry, will raise LDAP_Failure e. *)
val get_search_entry : conn -> msgid -> entry

(** Bind using simple authentication.  This is equivalent to calling bind_s
with auth_method `SIMPLE *)
val simple_bind_s : ?who:string -> ?password:string -> conn -> unit

(** Releases the ldap connection and frees associated resources.  The ldap
connection should not be used after it has been unbound. *)
val unbind : conn -> unit
